﻿using System;
using System.IO;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using System.Threading;
using System.Runtime.InteropServices;
using Microsoft.MediaCenter;
using Microsoft.MediaCenter.Hosting;
using Microsoft.MediaCenter.UI;

namespace MCE7RecordingLED
{
    public class MCE7RecordingLED : IAddInModule, IAddInEntryPoint
    {
        //x64
        [DllImport("inpoutx64.dll", EntryPoint = "Out32")]
        public static extern void Output(short address, short value);

        //x86
        //[DllImport("inpout32.dll", EntryPoint = "Out32")]
        //public static extern void Output(short address, short value);

        [DllImport("kernel32")]
        private static extern int GetPrivateProfileString(string section, string key, string def, StringBuilder retVal, int size, string filePath);

        private AddInHost mcHost;
        private string iniPath = "C:\\windows\\ehome\\mcerecordled.ini";
        private bool debug = false;
        private short portvalue = 0x1;
        private short portaddress = 0x378;
        private int tunercount = 0;
        private int running = 1;
        private string progtitle = "MCE7 Recording LED";
        private Microsoft.MediaCenter.TV.Scheduling.EventSchedule scheduler;

        public void Initialize(Dictionary<string, object> appInfo, Dictionary<string, object> entryPointInfo)
        {
        }

        public void Uninitialize()
        {
            running = 0;
            scheduler.ScheduleEventStateChanged -= this.onEventChange;
        }
     
        protected void onEventChange(object sender, Microsoft.MediaCenter.TV.Scheduling.ScheduleEventChangedEventArgs e)
        {
            //mcHost.MediaCenterEnvironment.Dialog("Event change", "Event Change Dialog", DialogButtons.Ok, 10, false);

            for (int i = 0; i < e.Changes.Length; i++)
            {
                /*
                if (debug)
                {
                    mcHost.MediaCenterEnvironment.Dialog("Schedule Event Change: id=" + e.Changes[i].ScheduleEventId + " previous=" + e.Changes[i].PreviousState.ToString() + " new=" + e.Changes[i].NewState.ToString(),
                        progtitle, DialogButtons.Ok, 10, false);
                }
                */
                if (e.Changes[i].PreviousState == Microsoft.MediaCenter.TV.Scheduling.ScheduleEventStates.IsOccurring)
                {
                    if (debug)
                    {
                        mcHost.MediaCenterEnvironment.Dialog("Tuner Stopped Recording: id=" + e.Changes[i].ScheduleEventId + " previous=" + e.Changes[i].PreviousState.ToString() + " new=" + e.Changes[i].NewState.ToString(),
                            progtitle, DialogButtons.Ok, 10, false);
                    }
                    if (tunercount > 0)
                    {
                        tunercount--;
                    }
                }
                if (e.Changes[i].NewState == Microsoft.MediaCenter.TV.Scheduling.ScheduleEventStates.IsOccurring)
                {
                    if (debug)
                    {
                        mcHost.MediaCenterEnvironment.Dialog("Tuner Started Recording: id=" + e.Changes[i].ScheduleEventId + " previous=" + e.Changes[i].PreviousState.ToString() + " new=" + e.Changes[i].NewState.ToString(),
                            progtitle, DialogButtons.Ok, 10, false);
                    }
                    tunercount++;
                }
            }

            if (tunercount > 0)
            {
                //turn on the LEDs
                Output(portaddress, portvalue);
            }
            else
            {
                //turn off the LEDs
                Output(portaddress, 0);
            }
        }

        void getIniParams()
        {
            string readvalue;

            if (!File.Exists(iniPath))
            {
                if (debug)
                {
                    mcHost.MediaCenterEnvironment.Dialog("No ini file found", progtitle, DialogButtons.Ok, 30, false);
                }
                return;
            }
            //load the params
            //debug
            readvalue = readIniKey("options", "debug");
            if (string.Compare(readvalue, "on", true) == 0)
            {
                debug = true;
            }
            //address
            readvalue = readIniKey("options", "portaddress");
            if (string.Compare(readvalue, "", true) != 0)
            {
                try
                {
                    portaddress = (short)int.Parse(readvalue);
                }
                catch (Exception e)
                {
                    string error = e.ToString();
                    portaddress = 0x378;
                }
            }

            //portvalue
            readvalue = readIniKey("options", "portvalue");
            if (string.Compare(readvalue, "", true) != 0)
            {
                try
                {
                    portvalue = (short)int.Parse(readvalue);
                }
                catch (Exception e)
                {
                    string error = e.ToString();
                    portaddress = 1;
                }
            }
        }

        public string readIniKey(string section, string key)
        {
            StringBuilder buf = new StringBuilder(1024);
            int i = GetPrivateProfileString(section, key, "", buf, 1024, iniPath);
            return buf.ToString();
        }

        public void Launch(AddInHost host)
        {
            mcHost = host;
            getIniParams();

            //initialize to off
            Output(portaddress, 0);
            
            try
            {
                scheduler = new Microsoft.MediaCenter.TV.Scheduling.EventSchedule();
                scheduler.ScheduleEventStateChanged += this.onEventChange;
            }
            catch (Microsoft.MediaCenter.TV.Scheduling.EventScheduleException e)
            {
                e.GetType(); //get rid of compiler warning
                mcHost.MediaCenterEnvironment.Dialog("MCE7led: no TV configured", progtitle, DialogButtons.Ok, 30, false);
            }
            catch (Exception e)
            {
                mcHost.MediaCenterEnvironment.Dialog("MCE7led error: " + e.ToString(), progtitle, DialogButtons.Ok, 30, false);
            }
            
            if (debug)
            {
                mcHost.MediaCenterEnvironment.Dialog("MCE7 Recording LED plugin loaded", progtitle, DialogButtons.Ok, 30, false);
            }

            while (running != 0)
            {
                Thread.Sleep(60000);
            }
        }
    }
}